@@ -1,9 +1,10 @@
 import shutil
+from asyncio import sleep as asyncio_sleep
 from contextlib import suppress
-from os import path
+from os import chdir, getcwd, listdir, path
 from types import SimpleNamespace
 
-from textual import events
+from textual import events, work
 from textual.app import App, ComposeResult
 from textual.binding import Binding
 from textual.containers import (
@@ -15,39 +16,36 @@ from textual.containers import (
 from textual.css.query import NoMatches
 from textual.widgets import Input
 
-from . import utils
-from .ActionButtons import (
+from rovr import utils
+from rovr.action_buttons import (
     CopyButton,
     CutButton,
     DeleteButton,
     NewItemButton,
     PasteButton,
     RenameItemButton,
-    SortOrderButton,
+    UnzipButton,
+    ZipButton,
 )
-from .FooterWidgets import (
-    Clipboard,
-    MetadataContainer,
-    ProcessContainer,
-)
-from .HeaderArea import HeaderArea
-from .maps import VAR_TO_DIR
-from .NavigationWidgets import (
-    BackButton,
-    ForwardButton,
-    PathAutoCompleteInput,
-    PathInput,
-    RefreshButton,
-    UpButton,
-)
-from .ScreensCore import YesOrNo, ZToDirectory
-from .themes import get_custom_themes
-from .utils import config
-from .WidgetsCore import (
+from rovr.core import (
     FileList,
     PinnedSidebar,
     PreviewContainer,
 )
+from rovr.footer import Clipboard, MetadataContainer, ProcessContainer
+from rovr.header import HeaderArea
+from rovr.maps import VAR_TO_DIR
+from rovr.navigation_widgets import (
+    BackButton,
+    ForwardButton,
+    PathAutoCompleteInput,
+    PathInput,
+    UpButton,
+)
+from rovr.screens import YesOrNo, ZDToDirectory
+from rovr.search_container import SearchInput
+from rovr.themes import get_custom_themes
+from rovr.utils import config
 
 
 class Application(App, inherit_bindings=False):
@@ -72,28 +70,25 @@ class Application(App, inherit_bindings=False):
         super().__init__(*args, **kwargs)
         self.prev_selected_option = None
         self.app_blurred = False
-        # TODO: need to actually do this.
-        self.main_sort_by = config["settings"]["filelist_sort_by"]
-        self.main_sort_order = config["settings"]["filelist_sort_order"]
 
     def compose(self) -> ComposeResult:
         print("Starting Rovr...")
         with Vertical(id="root"):
             yield HeaderArea(id="headerArea")
             with HorizontalScroll(id="menu"):
-                yield SortOrderButton()
                 yield CopyButton()
                 yield CutButton()
                 yield PasteButton()
                 yield NewItemButton()
                 yield RenameItemButton()
                 yield DeleteButton()
+                yield ZipButton()
+                yield UnzipButton()
             with VerticalGroup(id="below_menu"):
                 with HorizontalGroup():
                     yield BackButton()
                     yield ForwardButton()
                     yield UpButton()
-                    yield RefreshButton()
                     path_switcher = PathInput()
                     yield path_switcher
                 yield PathAutoCompleteInput(
@@ -101,40 +96,28 @@ class Application(App, inherit_bindings=False):
                 )
             with HorizontalGroup(id="main"):
                 with VerticalGroup(id="pinned_sidebar_container"):
+                    yield SearchInput(
+                        placeholder=f"({utils.get_icon('general', 'search')[0]}) Search"
+                    )
                     yield PinnedSidebar(id="pinned_sidebar")
                 with VerticalGroup(id="file_list_container"):
+                    yield SearchInput(
+                        placeholder=f"({utils.get_icon('general', 'search')[0]}) Search something..."
+                    )
                     yield FileList(
                         id="file_list",
                         name="File List",
                         classes="file-list",
-                        sort_by=self.main_sort_by,
-                        sort_order=self.main_sort_order,
                     )
                 yield PreviewContainer(
                     id="preview_sidebar",
                 )
             with HorizontalGroup(id="footer"):
-                # ? should we switch to a vertical scroll for richlog?
                 yield ProcessContainer()
                 yield MetadataContainer(id="metadata")
                 yield Clipboard(id="clipboard")
 
     def on_mount(self) -> None:
-        # this will stay until a beta release.
-        def warning(response: bool) -> None:
-            match response:
-                case True:
-                    pass
-                case False:
-                    self.app.exit(message="Bye! Hope to see you soon!")
-
-        self.push_screen(
-            YesOrNo(
-                "This is a pre-alpha application.\nUse at your own risk.\nContinue?",
-                reverse_color=True,
-            ),
-            warning,
-        )
         # border titles
         self.query_one("#menu").border_title = "Options"
         self.query_one("#below_menu").border_title = "Directory Actions"
@@ -152,11 +135,12 @@ class Application(App, inherit_bindings=False):
             self.query_one("#back").tooltip = "Go back in history"
             self.query_one("#forward").tooltip = "Go forward in history"
             self.query_one("#up").tooltip = "Go up the directory tree"
-            self.query_one("#refresh").tooltip = "Refresh the file list"
         self.tabWidget = self.query_one("Tabline")
         # make the file list
         self.query_one("#file_list").update_file_list()
         self.query_one("#file_list").focus()
+        # start watcher~
+        self.watch_for_changes_and_update()
 
     def action_focus_next(self) -> None:
         if config["settings"]["allow_tab_nav"]:
@@ -177,9 +161,7 @@ class Application(App, inherit_bindings=False):
         # Make sure that key binds don't break
         match event.key:
             # after input
-            case key if (
-                key in ["enter", "escape"] and self.focused.id == "path_switcher"
-            ):
+            case "enter" | "escape" if self.focused.id == "path_switcher":
                 await self.query_one(PathInput).action_submit()
                 self.query_one("#file_list").focus()
                 return
@@ -220,6 +202,8 @@ class Application(App, inherit_bindings=False):
                 elif self.query_one(PreviewContainer).display:
                     with suppress(NoMatches):
                         self.query_one("PreviewContainer > *").focus()
+                else:
+                    self.query_one("#file_list").focus()
             # Focus path switcher
             case key if key in config["keybinds"]["focus_toggle_path_switcher"]:
                 self.query_one("#path_switcher").focus()
@@ -244,12 +228,6 @@ class Application(App, inherit_bindings=False):
                     self.query_one("#file_list").focus()
                 elif self.query_one("#footer").display:
                     self.query_one("#clipboard").focus()
-            # toggle select mode
-            case key if (
-                key in config["keybinds"]["toggle_visual"]
-                and self.query_one("#file_list").has_focus
-            ):
-                await self.query_one("#file_list", FileList).toggle_mode()
             # Toggle hiding panels
             case key if key in config["keybinds"]["toggle_pinned_sidebar"]:
                 self.query_one("#file_list").focus()
@@ -275,20 +253,20 @@ class Application(App, inherit_bindings=False):
             ):
                 self.tabWidget.action_next_tab()
             case key if (
-                key in config["keybinds"]["tab_previous"]
-                and self.tabWidget.active_tab is not None
+                self.tabWidget.active_tab is not None
+                and key in config["keybinds"]["tab_previous"]
             ):
                 self.tabWidget.action_previous_tab()
             case key if key in config["keybinds"]["tab_new"]:
                 await self.tabWidget.add_tab(after=self.tabWidget.active_tab)
             case key if (
-                key in config["keybinds"]["tab_close"] and self.tabWidget.tab_count > 1
+                self.tabWidget.tab_count > 1 and key in config["keybinds"]["tab_close"]
             ):
                 await self.tabWidget.remove_tab(self.tabWidget.active_tab)
             # zoxide
             case key if (
-                event.key in config["plugins"]["zoxide"]["keybinds"]
-                and config["plugins"]["zoxide"]["enabled"]
+                config["plugins"]["zoxide"]["enabled"]
+                and event.key in config["plugins"]["zoxide"]["keybinds"]
             ):
                 if shutil.which("zoxide") is None:
                     self.notify(
@@ -298,7 +276,7 @@ class Application(App, inherit_bindings=False):
                     )
 
                 def on_response(response: str) -> None:
-                    """Handle the response from the ZToDirectory dialog."""
+                    """Handle the response from the ZDToDirectory dialog."""
                     if response:
                         pathinput = self.query_one(PathInput)
                         pathinput.value = utils.decompress(response).replace(
@@ -308,9 +286,12 @@ class Application(App, inherit_bindings=False):
                             SimpleNamespace(value=pathinput.value)
                         )
 
-                self.push_screen(ZToDirectory(), on_response)
+                self.push_screen(ZDToDirectory(), on_response)
             # zen mode
-            case key if key in config["plugins"]["zen_mode"]["keybinds"]:
+            case key if (
+                config["plugins"]["zen_mode"]["enabled"]
+                and key in config["plugins"]["zen_mode"]["keybinds"]
+            ):
                 if "zen" in self.classes:
                     self.remove_class("zen")
                 else:
@@ -322,5 +303,60 @@ class Application(App, inherit_bindings=False):
     def on_app_focus(self, event: events.AppFocus) -> None:
         self.app_blurred = False
 
+    @work
+    async def action_quit(self) -> None:
+        process_container = self.query_one(ProcessContainer)
+        if len(process_container.query("ProgressBarContainer")) != len(
+            process_container.query(".done")
+        ) + len(process_container.query(".error")) and not await self.push_screen_wait(
+            YesOrNo(
+                f"{len(process_container.query('ProgressBarContainer')) - len(process_container.query('.done')) - len(process_container.query('.error'))}"
+                + " processes are still running!\nAre you sure you want to quit?",
+                border_title="Quit [teal]rovr[/teal]",
+            )
+        ):
+            return
+        self.exit()
+
+    def cd(
+        self,
+        directory: str,
+        add_to_history: bool = True,
+        focus_on: str | None = None,
+    ) -> None:
+        if path.exists(directory):
+            if utils.normalise(getcwd()) == utils.normalise(directory):
+                self.query_one("#file_list").update_file_list(
+                    add_to_session=False, focus_on=focus_on
+                )
+                return
+            else:
+                chdir(directory)
+                self.query_one("#file_list").update_file_list(
+                    add_to_session=add_to_history, focus_on=focus_on
+                )
+        else:
+            while not path.exists(directory):
+                directory = "/".join(utils.normalise(directory).split("/")[:-1])
+            chdir(directory)
+            self.query_one("#file_list").update_file_list(
+                add_to_session=add_to_history, focus_on=focus_on
+            )
+
+    @work
+    async def watch_for_changes_and_update(self) -> None:
+        self._cwd = getcwd()
+        self._items = listdir(self._cwd)
+        while True:
+            await asyncio_sleep(1)
+            new_cwd = getcwd()
+            new_cwd_items = listdir(new_cwd)
+            if self._cwd != new_cwd:
+                self._cwd = new_cwd
+                self._items = listdir(self._cwd)
+            elif self._items != new_cwd_items:
+                self.cd(self._cwd)
+                self._items = new_cwd_items
+
 
 app = Application(watch_css=True)
