@@ -35,6 +35,7 @@ ASCII_ICONS = {
         "cut": ["X", "white"],
         "rename": ["R", "white"],
         "delete": ["D", "red"],
+        "zip": ["Z", "yellow"],
     },
     "folder": {
         "default": ["D", "#DAA520"],
@@ -69,6 +70,7 @@ ICONS = {
         "cut": ["\uf0c4", "white"],
         "rename": ["\uf246", "white"],
         "delete": ["\uf014", "red"],
+        "zip": ["\uf410", "yellow"],
     },
     "folder": {
         "default": ["\uf07b", "#DAA520"],
@@ -198,7 +200,7 @@ ICONS = {
         "xls": ["\uf1c3", "#27ae60"],
         "xml": ["\ue796", "#3498db"],
         "yml": ["\ue601", "#f39c12"],
-        "yaml": ["\ue8eb", "#f36c62"],
+        "*": ["\ue8eb", "#f36c62"],
         "zig": ["\ue6a9", "#9b59b6"],
         "zip": ["\uf410", "#e74c3c"],
     },
@@ -257,6 +259,8 @@ FILE_MAP = {
     ".wav": "audio",
     ".flac": "audio",
     ".ogg": "audio",
+    ".m4a": "audio",
+    ".opus": "audio",
     # Video files
     ".mp4": "video",
     ".avi": "video",
@@ -276,7 +280,11 @@ FILE_MAP = {
     ".zip": "zip",
     ".rar": "zip",
     ".tar": "zip",
-    ".gz": "zip",
+    ".tar.gz": "zip",
+    ".tgz": "zip",
+    ".tar.bz2": "zip",
+    ".tbz2": "zip",
+    ".tar.xz": "zip",
     ".7z": "zip",
     ".deb": "deb",
     ".rpm": "rpm",
@@ -432,6 +440,15 @@ FILES_MAP = {
     "py.typed": "py",
 }
 
+ARCHIVE_EXTENSIONS = [
+    ".zip",
+    ".tar",
+    ".gz",
+    ".bz2",
+    ".xz",
+    ".7z",
+]
+
 PIL_EXTENSIONS = [
     # reminder that stuff can also not work, just remove it if it doesnt work
     ".avif",
