@@ -30,25 +30,6 @@ lzstring = LZString()
 pprint = Console().print
 
 
-# What is textual reactive?
-class SessionManager:
-    """Manages session-related variables.
-
-    Attributes:
-        sessionDirectories (list[dict]): A list of dictionaries that contain a directory's name within.
-            The closer it is to index 0, the older it is.
-        sessionHistoryIndex (int): The index of the session in the sessionDirectories.
-            This can be a number between 0 and the length of the list - 1, inclusive.
-        sessionLastHighlighted (dict[str, int]): A dictionary mapping directory paths to the index of the
-            last highlighted item. If a directory is not in the dictionary, the default is 0.
-    """
-
-    def __init__(self) -> None:
-        self.sessionDirectories = []
-        self.sessionHistoryIndex = 0
-        self.sessionLastHighlighted = {}
-
-
 config = {}
 pins = {}
 
@@ -101,13 +82,12 @@ def open_file(filepath: str) -> None:
         print(f"Error opening file: {e}")
 
 
-def get_cwd_object(cwd: str, sort_order: str, sort_by: str) -> (list[dict], list[dict]):
+def get_cwd_object(cwd: str) -> (list[dict], list[dict]):
     """
     Get the objects (files and folders) in a provided directory
     Args:
         cwd(str): The working directory to check
-        sort_order(str): The sort order (ascending or descending)
-        sort_by(str): How to sort it (currently unused)
+
     Returns:
         folders(list[dict]): A list of dictionaries, containing "name" as the item's name and "icon" as the respective icon
         files(list[dict]): A list of dictionaries, containing "name" as the item's name and "icon" as the respective icon
@@ -132,8 +112,8 @@ def get_cwd_object(cwd: str, sort_order: str, sort_by: str) -> (list[dict], list
                 "dir_entry": item,
             })
     # Sort folders and files properly
-    folders.sort(key=lambda x: x["name"].lower(), reverse=(sort_order == "descending"))
-    files.sort(key=lambda x: x["name"].lower(), reverse=(sort_order == "descending"))
+    folders.sort(key=lambda x: x["name"].lower())
+    files.sort(key=lambda x: x["name"].lower())
     print(f"Found {len(folders)} folders and {len(files)} files in {cwd}")
     return folders, files
 
@@ -165,17 +145,51 @@ def file_is_type(file_path: str) -> str:
         return "file"
 
 
-def get_recursive_files(object_path: str) -> list[str]:
+def force_obtain_write_permission(item_path: str) -> bool:
+    """
+    Forcefully obtain write permission to a file or directory.
+
+    Args:
+        item_path (str): The path to the file or directory.
+
+    Returns:
+        bool: True if permission was granted successfully, False otherwise.
+    """
+    if not path.exists(item_path):
+        return False
+    try:
+        current_permissions = stat.S_IMODE(os.lstat(item_path).st_mode)
+        os.chmod(item_path, current_permissions | stat.S_IWRITE)
+        return True
+    except (OSError, PermissionError) as e:
+        pprint(
+            f"[bright_red]Permission Error:[/] Failed to change permission for {item_path}: {e}"
+        )
+        return False
+
+
+def get_recursive_files(object_path: str, with_folders: bool = False) -> list[dict]:
     """Get the files available at a directory recursively, regardless of whether it is a directory or not
     Args:
         object_path (str): The object's path
+        with_folders (bool): Return a list of folders as well
 
     Returns:
         list: A list of dictionaries, with a "path" key and "relative_loc" key
+        OR
+        list: A list of dictionaries, with a "path" key and "relative_loc" key for files
+        list: A list of path strings that were involved in the file list.
     """
     if path.isfile(path.realpath(object_path)) or path.islink(
         path.realpath(object_path)
     ):
+        if with_folders:
+            return [
+                {
+                    "path": normalise(object_path),
+                    "relative_loc": path.basename(object_path),
+                }
+            ], []
         return [
             {
                 "path": normalise(object_path),
@@ -184,20 +198,23 @@ def get_recursive_files(object_path: str) -> list[str]:
         ]
     else:
         files = []
-        for folder, _, files_in_folder in os.walk(object_path):
+        folders = []
+        for folder, folders_in_folder, files_in_folder in os.walk(object_path):
+            if with_folders:
+                for folder_in_folder in folders_in_folder:
+                    full_path = normalise(path.join(folder, folder_in_folder))
+                    if full_path not in folder:
+                        folders.append(full_path)
             for file in files_in_folder:
                 full_path = normalise(path.join(folder, file))  # normalise the path
-                if (
-                    normalise(path.realpath(full_path)) != full_path
-                ):  # ie we passed over a symlink
-                    pass  # will hopefully be taken by shutil.rmtree
-                else:
-                    files.append({
-                        "path": full_path,
-                        "relative_loc": normalise(
-                            path.relpath(full_path, object_path + "/..")
-                        ),
-                    })
+                files.append({
+                    "path": full_path,
+                    "relative_loc": normalise(
+                        path.relpath(full_path, object_path + "/..")
+                    ),
+                })
+        if with_folders:
+            return files, folders
         return files
 
 
@@ -295,6 +312,41 @@ def deep_merge(d: dict, u: dict) -> dict:
     return d
 
 
+def set_nested_value(d: dict, path_str: str, value: bool) -> None:
+    """Sets a value in a nested dictionary using a dot-separated path string.
+
+    Args:
+        d (dict): The dictionary to modify.
+        path_str (str): The dot-separated path to the key (e.g., "plugins.zen_mode").
+        value (bool): The value to set. (boolean for now)
+    """
+    keys = path_str.split(".")
+    current = d
+    for i, key in enumerate(keys):
+        if i == len(keys) - 1:
+            try:
+                if isinstance(current[key], dict) and "enabled" in current[key]:
+                    current[key]["enabled"] = value
+                elif type(current[key]) is type(value):
+                    current[key] = value
+                else:
+                    pprint("[bright_red][underline]Config Error:[/]")
+                    pprint(
+                        f"[cyan][b]{path_str}[/b][/cyan]'s new value of type [cyan][b]{type(value).__name__}[/b][/cyan] is not a [cyan][b]{type(current[key]).__name__}[/b][/cyan] type, and cannot be modified."
+                    )
+                    exit(1)
+            except KeyError:
+                pprint("[bright_red][underline]Config Error:[/]")
+                pprint(
+                    f"[cyan][b]{path_str}[/b][/cyan] is not a valid path to an existing value and hence cannot be set."
+                )
+                exit(1)
+        else:
+            if not isinstance(current.get(key), dict):
+                current[key] = {}
+            current = current[key]
+
+
 def load_config() -> None:
     """
     Load both the template config and the user config
